﻿
//==================================================================

#target indesign
if (app.documents.length == 0) ErrorExit("Please open a document and try again.");

const gScriptName = "Export from InDesign to Photoshop"; // Name of the script
const gScriptVersion = "1.1"; // Version

var gDoc = app.activeDocument;
var gActPage = app.activeWindow.activePage;

var docPath = gDoc.filePath.absoluteURI + "/";
var gTempFolderPath = docPath + 'export/';
var gTempFolder = new Folder(gTempFolderPath);

if (!gTempFolder.exists) gTempFolder.create();

var myPageName = gActPage.name;    
app.pdfExportPreferences.pageRange = myPageName;

var pdfPresetName = "ExportToPS_bleedmarks";
var pdfPreset = app.pdfExportPresets.itemByName(pdfPresetName);
if (!pdfPreset.isValid) ErrorExit("Es muss noch eine Voreinstellung für den PDF-Export mit dem Namen \"" + pdfPresetName + "\" unter 'Datei > Adobe PDF Vorgaben' angelegt werden.", true);


Main();

function Main() {

	var selectedItems = app.selection;
	if (!selectedItems.length) ErrorExit("Keine Elemente ausgewählt!");

	var newLayer = gDoc.layers.add();

	for (var i = 0; i < selectedItems.length; i++) {

		var item = selectedItems[i];
		
		var itemHeight = Math.ceil(item.geometricBounds[2]-item.geometricBounds[0]);
		var itemWidth = Math.ceil(item.geometricBounds[3]-item.geometricBounds[1]);

		var itemX = item.geometricBounds[1];
		var itemY = item.geometricBounds[0];

		var pageWidth = gActPage.bounds[3];
		var pageHeight = gActPage.bounds[2];

		var clone = item.duplicate(newLayer);
	}

	gDoc.layers.everyItem().visible = false;
	newLayer.visible = true;

	pdfFileName = "tmp-" + str_pad(i+1) + ".pdf";
	pdfFilePath = gTempFolderPath + pdfFileName;
	pdfFile = new File(pdfFilePath);
	gDoc.exportFile(ExportFormat.PDF_TYPE, pdfFile, false, pdfPreset);

	PlacePDFinPSD( pdfFilePath );

	gDoc.layers.everyItem().visible = true;
	newLayer.remove();


/*
		var newDoc = app.documents.add();

		// https://github.com/fabiantheblind/extendscript/wiki/Pages-And-Margins
		newDoc.pages.item(0).marginPreferences.properties = {right:0, left:0, bottom:0, top:0};
		newDoc.viewPreferences.horizontalMeasurementUnits = gDoc.viewPreferences.horizontalMeasurementUnits;
		newDoc.viewPreferences.verticalMeasurementUnits = gDoc.viewPreferences.verticalMeasurementUnits;

		// http://jongware.mit.edu/idcs4js/pc_DocumentPreference.html
		newDoc.documentPreferences.properties = {
			pageHeight: pageHeight+'px',
			pageWidth: pageWidth+'px',
			pageOrientation: pageWidth > pageHeight ? PageOrientation.LANDSCAPE : PageOrientation.PORTRAIT,
		}

		// http://www.adobe.com/content/dam/Adobe/en/devnet/indesign/cs5_docs/indesign_scripting/guide-en-US/InDesignCS5_ScriptingGuide_JS.pdf
		newDoc.pages.item(0).resize(
			CoordinateSpaces.INNER_COORDINATES,
			AnchorPoint.CENTER_ANCHOR,
			ResizeMethods.REPLACING_CURRENT_DIMENSIONS_WITH,
			[pageWidth, pageHeight]
		);
		


		var clone = item.duplicate(newDoc.layers[0]);
		//clone.geometricBounds = [0,0,itemHeight,itemWidth];
		clone.move([itemX,itemY]);


		pdfFileName = "tmp-" + str_pad(i+1) + ".pdf";
		pdfFilePath = gTempFolderPath + pdfFileName;
		pdfFile = new File(pdfFilePath);
		newDoc.exportFile(ExportFormat.PDF_TYPE, pdfFile, false, pdfPreset);
		newDoc.close(SaveOptions.NO);

		PlacePDFinPSD( pdfFilePath );
*/


}

function PlacePDFinPSD(filePath) {
	var bt = new BridgeTalk();
	bt.target = "photoshop";
	var script = PlacePdf.toString() + '\r';
	script += 'PlacePdf("' + filePath + '");';
	bt.body = script;
//~ 	$.write(script);
//~ 	exit();
	bt.onResult = function(resObj) {} 
	bt.onError = function(resObj) {
		alert(resObj.body);
	}
	bt.send(100);
}

//--------------------------------------------------------------------------------------------------------------------------------------------------------
function PlacePdf(filePath) {
	var idPlc = charIDToTypeID( "Plc " );
	var desc2 = new ActionDescriptor();
	var idAs = charIDToTypeID( "As  " );
	var desc3 = new ActionDescriptor();
	var idfsel = charIDToTypeID( "fsel" );
	var idpdfSelection = stringIDToTypeID( "pdfSelection" );
	var idpage = stringIDToTypeID( "page" );
	desc3.putEnumerated( idfsel, idpdfSelection, idpage );
	var idPgNm = charIDToTypeID( "PgNm" );
	desc3.putInteger( idPgNm, 1 );
	var idCrop = charIDToTypeID( "Crop" );
	var idcropTo = stringIDToTypeID( "cropTo" );
	var idcropBox = stringIDToTypeID( "cropBox" );
	desc3.putEnumerated( idCrop, idcropTo, idcropBox );
	var idPDFG = charIDToTypeID( "PDFG" );
	desc2.putObject( idAs, idPDFG, desc3 );
	var idnull = charIDToTypeID( "null" );
	desc2.putPath( idnull, new File( filePath ) );
	var idFTcs = charIDToTypeID( "FTcs" );
	var idQCSt = charIDToTypeID( "QCSt" );
	var idQcsa = charIDToTypeID( "Qcsa" );
	desc2.putEnumerated( idFTcs, idQCSt, idQcsa );
	var idOfst = charIDToTypeID( "Ofst" );
	var desc4 = new ActionDescriptor();
	var idHrzn = charIDToTypeID( "Hrzn" );
	var idRlt = charIDToTypeID( "#Rlt" );
	desc4.putUnitDouble( idHrzn, idRlt, 0.000000 );
	var idVrtc = charIDToTypeID( "Vrtc" );
	var idRlt = charIDToTypeID( "#Rlt" );
	desc4.putUnitDouble( idVrtc, idRlt, 0.000000 );
	var idOfst = charIDToTypeID( "Ofst" );
	desc2.putObject( idOfst, idOfst, desc4 );
	var idAntA = charIDToTypeID( "AntA" );
	desc2.putBoolean( idAntA, true );
	try {
		executeAction( idPlc, desc2, DialogModes.NO );
		//app.activeDocument.activeLayer.rasterize(RasterizeType.ENTIRELAYER);
	}
	catch(err) {
		$.writeln(err);
	}
}


function var_dump (obj) {
	var str = "";
	for (var j in obj) {
		str += j + "\n";
	}
	alert(str);
}

function SetDisplayDialogs(mode) {
	var bt = new BridgeTalk;
	bt.target = "photoshop";
	var script = 'app.displayDialogs = DialogModes.' + mode + ';';
	bt.body = script;
	bt.send();
}

function str_pad ( str ) {
	var str = "" + str;
	var pad = "000";
	return pad.substring(0, pad.length - str.length) + str;	
}

function ErrorExit(error, icon) {
	alert(error, gScriptName + " - " + gScriptVersion, icon);
	exit();
}


