﻿
//==================================================================

#target indesign
if (app.documents.length == 0) ErrorExit("Please open a document and try again.");

var gSourceDPI = 72;
var gTargetDPI = 160;
var gRatioDPI = gSourceDPI/gTargetDPI;

const gScriptName = "Export from InDesign to Photoshop"; // Name of the script
const gScriptVersion = "1.1"; // Version

var gDoc = app.activeDocument;
var gActPage = app.activeWindow.activePage;

var docPath = gDoc.filePath.absoluteURI + "/";
var gTempFolderPath = docPath + 'export/';
var gTempFolder = new Folder(gTempFolderPath);

if (!gTempFolder.exists) gTempFolder.create();

var myPageName = gActPage.name;    
app.pdfExportPreferences.pageRange = myPageName;

var pdfPresetName = "ExportToPS_bleedmarks";
var pdfPreset = app.pdfExportPresets.itemByName(pdfPresetName);
if (!pdfPreset.isValid) ErrorExit("Es muss noch eine Voreinstellung für den PDF-Export mit dem Namen \"" + pdfPresetName + "\" unter 'Datei > Adobe PDF Vorgaben' angelegt werden.", true);



Main();

function Main() {

	var str = "";

	var selectedItems = app.selection;
	if (!selectedItems.length) ErrorExit("Keine Elemente ausgewählt!");

	for (var i = 0; i < selectedItems.length; i++) {

		var item = selectedItems[i];
		if (item instanceof TextFrame) {
			
			SetDisplayDialogs("NO");
			var textObj = GetTextData(item);
			var serObj = textObj.toSource();
			str = serObj;
			CreateBridgeTalkMessage(serObj);
			SetDisplayDialogs("YES");

		}

		/*
		var itemHeight = Math.ceil(item.geometricBounds[2]-item.geometricBounds[0]);
		var itemWidth = Math.ceil(item.geometricBounds[3]-item.geometricBounds[1]);

		var itemX = item.geometricBounds[1];
		var itemY = item.geometricBounds[0];

		var pageWidth = gActPage.bounds[3];
		var pageHeight = gActPage.bounds[2];

		var newLayer = gDoc.layers.add();
		var clone = item.duplicate(newLayer);

		gDoc.layers.everyItem().visible = false;
		newLayer.visible = true;

		pdfFileName = "tmp-" + str_pad(i+1) + ".pdf";
		pdfFilePath = gTempFolderPath + pdfFileName;
		pdfFile = new File(pdfFilePath);
		gDoc.exportFile(ExportFormat.PDF_TYPE, pdfFile, false, pdfPreset);

		PlacePDFinPSD( pdfFilePath );

		gDoc.layers.everyItem().visible = true;
		newLayer.remove();
		*/

	}

}

function calc_target_size ( n ) {
    return Math.ceil( n * gRatioDPI );
}

//--------------------------------------------------------------------------------------------------------------------------------------------------------
function GetTextData(tf) {
	var obj, tsr;
	var textRanges = [];
	var txtObj = {}
	var doc = app.activeDocument;
	var docPref = doc.documentPreferences;
	var topSlug = docPref.slugTopOffset;
	var leftSlug = docPref.slugInsideOrLeftOffset;
	var gb = tf.geometricBounds;
	var width = (gb[3] - gb[1]);
	var height = (gb[2] - gb[0]);
	var txt = tf.texts[0];
	var cont = txt.contents.replace(/(\r|\n)/g, "_");
	var tsrs = txt.textStyleRanges;
	var blending = tf.properties.transparencySettings.properties.blendingSettings;
    
	for (var i = 0; i < tsrs.length; i++) {
		tsr = tsrs[i];
		
		var col = tsr.fillColor.colorValue;
		if (col[0]+col[1]+col[2]+col[3] == 0) col = [255,255,255,100];

		obj = {
			contents : tsr.contents.replace(/(\r|\n)/g, "_"),
			color : col,
			font : tsr.appliedFont.postscriptName,
			letterSpacing: tsr.tracking,
             leading: tsr.leading,
			size : (tsr.pointSize),
			length : tsr.characters.length,
			from : tsr.characters.firstItem().index,
			to : tsr.characters.lastItem().index +1
		}
		
		textRanges.push(obj);
	}

	

	txtObj.textRanges = textRanges;
//	txtObj.position = [ gb[1]+7.4, gb[0]+7.8 ];
	txtObj.position = [ gb[1], gb[0]];
	txtObj.frameWidth = width;
	txtObj.frameHeight = height;
	txtObj.contents = cont;
	txtObj.layerName = cont.substr(0,20);
	txtObj.opacity = blending.opacity;
    
	switch(txt.justification) {
		case Justification.LEFT_ALIGN:
			txtObj.justification = "Justification.LEFT";
			break;
		case Justification.CENTER_ALIGN:
			txtObj.justification = "Justification.CENTER";
			break;
		case Justification.RIGHT_ALIGN:
			txtObj.justification = "Justification.RIGHT";
			break;
		case Justification.LEFT_JUSTIFIED:
			txtObj.justification = "Justification.LEFTJUSTIFIED";
			break;		
		case Justification.RIGHT_JUSTIFIED:
			txtObj.justification = "Justification.RIGHTJUSTIFIED";
			break;
		case Justification.CENTER_JUSTIFIED:
			txtObj.justification = "Justification.CENTERJUSTIFIED";
			break;
		case Justification.FULLY_JUSTIFIED:
			txtObj.justification = "Justification.FULLYJUSTIFIED";
			break;			
		default:
			txtObj.justification = "Justification.LEFT";
	}
	
	return txtObj;
}

//============================== FUNCTIONS  EDITABLE TEXT =================================
function CreateBridgeTalkMessage(serObj) {
	var bt = new BridgeTalk();
	bt.target = "photoshop";
	
	var script = '//@includepath "/c/Program Files/Adobe/xtools;/Developer/xtools;/Applications/Adobe Photoshop CS6/xtools"\r';
	script += '//@include "xlib/stdlib.js"\r';
	script += '//@include "xlib/Text.jsx"\r';
	script += 'ProcessInPS = ' + ProcessInPS.toSource() + '\r';
     script += 'gRatioDPI = ' + gRatioDPI + '\r';
     script += 'calc_target_size = ' + calc_target_size.toSource() + '\r';
	script += 'ProcessInPS(' + serObj + ');';
	bt.body = script;
//~ 	$.write(script);
//~ 	exit();
	bt.onError = function(errObj) {
		$.writeln("Error: " + errObj.body);
	}
	bt.onResult = function(resObj) {} 
	bt.send(100);
}

//--------------------------------------------------------------------------------------------------------------------------------------------------------
function ProcessInPS(obj) {
	var doc, tsr, s;
	var r = String.fromCharCode(13);
	var tsrs = obj.textRanges;
	tsr = tsrs[0];
	var opts = new TextOptions(obj.contents.replace(/_/g, r));
	var width = new UnitValue(obj.frameWidth,'px');
	var height = new UnitValue(obj.frameHeight,'px');
	if (app.documents.length == 0) {
		doc = app.documents.add(width, height, 72, "Text test", NewDocumentMode.RGB);
	}
	else {
		doc = app.activeDocument;
	}
	var textLayer = doc.artLayers.add();
	textLayer.kind = LayerKind.TEXT;
    textLayer.opacity = obj.opacity;
    
	var solidColorRef = new SolidColor();
	solidColorRef.rgb.red = tsr.color[0];
	solidColorRef.rgb.green = tsr.color[1];
	solidColorRef.rgb.blue = tsr.color[2];

	var ti = textLayer.textItem;  
	ti.contents = obj.contents.replace(/_/g, r);
	ti.kind = TextType.PARAGRAPHTEXT;
	ti.width = calc_target_size(width);
	ti.height = calc_target_size(height);
	ti.position = [new UnitValue(obj.position[0], 'px'), new UnitValue(obj.position[1], 'px')];
	ti.font = tsr.font;
	ti.size = new UnitValue(calc_target_size(tsr.size), 'pt');
	ti.color = solidColorRef;
	ti.justification = eval(obj.justification);
	textLayer.name = obj.layerName;
	var ranges = new TextStyleRanges();
	
	var t = "";

	for (var i = 0; i < tsrs.length; i++) {
		tsr = tsrs[i];
		tsr.contents = tsr.contents.replace(/_/g, r);
		t = tsr;
		s = new TextStyle(tsr.font, new UnitValue(calc_target_size(tsr.size), 'pt'), Text.toRGBColor(tsr.color[0], tsr.color[1], tsr.color[2]), calc_target_size(tsr.letterSpacing), calc_target_size(tsr.leading));
		ranges.add(new TextStyleRange(tsr.from, tsr.to, s));	
	}

    ti.antiAliasMethod = AntiAlias.SHARP;
    ti.autoKerning = AutoKernType.OPTICAL;
   
	opts.ranges = ranges;
	Text.modifyTextLayer(doc, opts, doc.activeLayer);
}

//--------------------------------------------------------------------------------------------------------------------------------------------------------

function PlacePDFinPSD(filePath) {
	var bt = new BridgeTalk();
	bt.target = "photoshop";
	var script = PlacePdf.toString() + '\r';
	script += 'PlacePdf("' + filePath + '");';
	bt.body = script;
//~ 	$.write(script);
//~ 	exit();
	bt.onResult = function(resObj) {} 
	bt.onError = function(resObj) {
		alert(resObj.body);
	}
	bt.send(100);
}

//--------------------------------------------------------------------------------------------------------------------------------------------------------
function PlacePdf(filePath) {
	var idPlc = charIDToTypeID( "Plc " );
	var desc2 = new ActionDescriptor();
	var idAs = charIDToTypeID( "As  " );
	var desc3 = new ActionDescriptor();
	var idfsel = charIDToTypeID( "fsel" );
	var idpdfSelection = stringIDToTypeID( "pdfSelection" );
	var idpage = stringIDToTypeID( "page" );
	desc3.putEnumerated( idfsel, idpdfSelection, idpage );
	var idPgNm = charIDToTypeID( "PgNm" );
	desc3.putInteger( idPgNm, 1 );
	var idCrop = charIDToTypeID( "Crop" );
	var idcropTo = stringIDToTypeID( "cropTo" );
	var idcropBox = stringIDToTypeID( "cropBox" );
	desc3.putEnumerated( idCrop, idcropTo, idcropBox );
	var idPDFG = charIDToTypeID( "PDFG" );
	desc2.putObject( idAs, idPDFG, desc3 );
	var idnull = charIDToTypeID( "null" );
	desc2.putPath( idnull, new File( filePath ) );
	var idFTcs = charIDToTypeID( "FTcs" );
	var idQCSt = charIDToTypeID( "QCSt" );
	var idQcsa = charIDToTypeID( "Qcsa" );
	desc2.putEnumerated( idFTcs, idQCSt, idQcsa );
	var idOfst = charIDToTypeID( "Ofst" );
	var desc4 = new ActionDescriptor();
	var idHrzn = charIDToTypeID( "Hrzn" );
	var idRlt = charIDToTypeID( "#Rlt" );
	desc4.putUnitDouble( idHrzn, idRlt, 0.000000 );
	var idVrtc = charIDToTypeID( "Vrtc" );
	var idRlt = charIDToTypeID( "#Rlt" );
	desc4.putUnitDouble( idVrtc, idRlt, 0.000000 );
	var idOfst = charIDToTypeID( "Ofst" );
	desc2.putObject( idOfst, idOfst, desc4 );
	var idAntA = charIDToTypeID( "AntA" );
	desc2.putBoolean( idAntA, true );
	try {
		executeAction( idPlc, desc2, DialogModes.NO );
		//app.activeDocument.activeLayer.rasterize(RasterizeType.ENTIRELAYER);
	}
	catch(err) {
		$.writeln(err);
	}
}


function var_dump (obj) {

	var str = "";
	for (var j in obj) {
		str += j + " = " + (obj[j]+"").substr(0,20) + "\n";
	}
	alert(str);
	$.write(str);
}

function SetDisplayDialogs(mode) {
	var bt = new BridgeTalk;
	bt.target = "photoshop";
	var script = 'app.displayDialogs = DialogModes.' + mode + ';';
	bt.body = script;
	bt.send();
}

function str_pad ( str ) {
	var str = "" + str;
	var pad = "000";
	return pad.substring(0, pad.length - str.length) + str;	
}

function ErrorExit(error, icon) {
	alert(error, gScriptName + " - " + gScriptVersion, icon);
	exit();
}


