﻿
//==================================================================

#target indesign

if (app.documents.length == 0) ErrorExit("Please open a document and try again.");

const gScriptName = "Export from InDesign to Photoshop"; // Name of the script
const gScriptVersion = "1.1"; // Version

var gUeberfuellung = 5;
var gExportSVGs = true;
var gExportPNGs = [1,2,4];

var gDoc = app.activeDocument;
var gActPage = app.activeWindow.activePage;

var docPath = gDoc.filePath.absoluteURI + "/";
var gTempFolderPath = docPath + 'export/';
var gTempFolder = new Folder(gTempFolderPath);

if (!gTempFolder.exists) gTempFolder.create();

var myPageName = gActPage.name;    
app.pdfExportPreferences.pageRange = myPageName;

var pdfPresetName = "ExportToPS_bleedmarks";
var pdfPreset = app.pdfExportPresets.itemByName(pdfPresetName);
if (!pdfPreset.isValid) ErrorExit("Es muss noch eine Voreinstellung für den PDF-Export mit dem Namen \"" + pdfPresetName + "\" unter 'Datei > Adobe PDF Vorgaben' angelegt werden.", true);


Main();


function Main() {

	var itemsByName = {};

	var selectedItems = app.selection;
	if (!selectedItems.length) ErrorExit("Keine Elemente ausgewählt!");

	var pdfFiles = [];

	for (var i = 0; i < selectedItems.length; i++) {
		var item = selectedItems[i];

		var newDoc = app.documents.add();

		var itemHeight = Math.ceil(item.visibleBounds[2]-item.visibleBounds[0]);
		var itemWidth = Math.ceil(item.visibleBounds[3]-item.visibleBounds[1]);
		
		// https://github.com/fabiantheblind/extendscript/wiki/Pages-And-Margins
		newDoc.pages.item(0).marginPreferences.properties = {right:0, left:0, bottom:0, top:0};
		newDoc.viewPreferences.horizontalMeasurementUnits = gDoc.viewPreferences.horizontalMeasurementUnits;
		newDoc.viewPreferences.verticalMeasurementUnits = gDoc.viewPreferences.verticalMeasurementUnits;

		// http://jongware.mit.edu/idcs4js/pc_DocumentPreference.html
		newDoc.documentPreferences.properties = {
			pageHeight: '100px',
			pageWidth: '100px',
			pageOrientation: itemWidth > itemHeight ? PageOrientation.LANDSCAPE : PageOrientation.PORTRAIT,
		}

		// http://www.adobe.com/content/dam/Adobe/en/devnet/indesign/cs5_docs/indesign_scripting/guide-en-US/InDesignCS5_ScriptingGuide_JS.pdf
		newDoc.pages.item(0).resize(
			CoordinateSpaces.INNER_COORDINATES,
			AnchorPoint.CENTER_ANCHOR,
			ResizeMethods.REPLACING_CURRENT_DIMENSIONS_WITH,
			[itemWidth+gUeberfuellung*2, itemHeight+gUeberfuellung*2]
		);
		

		var clone = item.duplicate(newDoc.layers[0]);
		//clone.geometricBounds = [0,0,itemHeight,itemWidth];
		clone.move([gUeberfuellung,gUeberfuellung]);

		var name = item.name;
		if (name == "" || name == null) name = str_pad(item.id);

		if (itemsByName[name] == null) itemsByName[name] = 0;
		itemsByName[name]++;
		if (itemsByName[name] > 1) name += '-'+(itemsByName[name]-1);


		pdfFileName = name + ".pdf";
		pdfFilePath = gTempFolderPath + pdfFileName;
		pdfFile = new File(pdfFilePath);
		newDoc.exportFile(ExportFormat.PDF_TYPE, pdfFile, false, pdfPreset);
		pdfFiles.push(pdfFilePath);
		
/*
		svgFileName = str_pad(i+1) + ".svg";
		svgFilePath = gTempFolderPath + svgFileName;
		svgFile = new File(svgFilePath);
		newDoc.exportFile(ExportFormat.SVG, svgFile, false, false);
*/
		newDoc.close(SaveOptions.NO);

	}

	if (gExportSVGs || gExportPNGs.length) {
		SetDisplayDialogs("NO");
		var obj = {files:pdfFiles, svg:gExportSVGs, png:gExportPNGs};
		BridgeTalk.bringToFront("illustrator");
		CreateBridgeTalkMessage(obj.toSource());
		BridgeTalk.bringToFront("indesign");
		SetDisplayDialogs("YES");
	}

}

function CreateBridgeTalkMessage(serObj) {

	var bt = new BridgeTalk();
	bt.target = "illustrator";

	var script = "";	
	script += 'ProcessInIllustrator = ' + ProcessInIllustrator.toSource() + '\r';
	script += 'ProcessInIllustrator(' + serObj + ');';
	bt.body = script;
	bt.onError = function(errObj) {
		$.writeln("Error: " + errObj.body);
	}
	bt.onResult = function(resObj) {} 
	bt.send(100);
}

// http://wwwimages.adobe.com/content/dam/Adobe/en/devnet/pdf/illustrator/scripting/cs6/Illustrator-Scripting-Reference-JavaScript.pdf

function ProcessInIllustrator ( opt ) {

	var pdfFilePath = opt.files;
	
	var options = new ExportOptionsSVG();
	options.fontSubsetting = SVGFontSubsetting.GLYPHSUSED;
	options.embedRasterImages = false;
	options.fontType = SVGFontType.OUTLINEFONT;
	options.artBoardClipping = true;

	var pngExportOpts = new ExportOptionsPNG24();
    pngExportOpts.antiAliasing = true;
    pngExportOpts.artBoardClipping = false;
    pngExportOpts.saveAsHTML = false;
    pngExportOpts.transparency = true;
	
	var pngSizes = opt.png;
	
	for (var i = 0; i < pdfFilePath.length; i++) {
		
		var pdfFile = new File(pdfFilePath[i]);
		open(pdfFile, DocumentColorSpace.RGB);
		
		var curDoc = app.activeDocument;
		
		var f = pdfFilePath[i].split('.pdf');
		f.pop();

		if (opt.svg) {
			var file = new File( f.join('') + '.svg');
			curDoc.exportFile(file, ExportType.SVG, options);
		}
	
		if (pngSizes.length) {
			for (var s in pngSizes) {
				var size= pngSizes[s];
				file = new File( f.join('') + '_x' + size + '.png');
				pngExportOpts.verticalScale = 100 * size;
				pngExportOpts.horizontalScale = 100 * size;
				curDoc.exportFile(file, ExportType.PNG24, pngExportOpts);
			}
		}		

		curDoc.close( SaveOptions.DONOTSAVECHANGES );
	}

}

function SetDisplayDialogs(mode) {
	var bt = new BridgeTalk;
	bt.target = "illustrator";
	var script = 'app.displayDialogs = DialogModes.' + mode + ';';
	bt.body = script;
	bt.send();
}


function str_pad ( str ) {
	var str = "" + str;
	var pad = "00000";
	return pad.substring(0, pad.length - str.length) + str;	
}

function ErrorExit(error, icon) {
	alert(error, gScriptName + " - " + gScriptVersion, icon);
	exit();
}


