﻿
//==================================================================

#target indesign
if (app.documents.length == 0) ErrorExit("Please open a document and try again.");

const gScriptName = "Export from InDesign to Photoshop"; // Name of the script
const gScriptVersion = "1.1"; // Version

var gDoc = app.activeDocument;
var gActPage = app.activeWindow.activePage;

var gTagLayer = gDoc.layers.itemByName("Textfelder-IDs");
if (gTagLayer != null) gTagLayer.remove();

gTagLayer = gDoc.layers.add();
gTagLayer.name = "Textfelder-IDs";
gTagLayer.move(LocationOptions.AT_BEGINNING);

var gTagColor = gDoc.colors.itemByName("text_ids");
if (gTagColor != null) gTagColor.remove();

gTagColor = gDoc.colors.add();
gTagColor.space = ColorSpace.CMYK;
gTagColor.model = ColorModel.PROCESS;
gTagColor.name = "text_ids";
gTagColor.colorValue = [0,100,0,30];

var gLabelColor = gDoc.colors.itemByName("text_ids_label");
if (gLabelColor != null) gLabelColor.remove();

gLabelColor = gDoc.colors.add();
gLabelColor.space = ColorSpace.CMYK;
gLabelColor.model = ColorModel.PROCESS;
gLabelColor.name = "text_ids_label";
gLabelColor.colorValue = [0,0,0,0];


//var docPath = gDoc.filePath.absoluteURI + "/";

var gTypeSheetDimensions = {width: '900', height:'900'};

var gSourceDPI = 72;
var gTargetDPI = 160;
var gRatioDPI = gSourceDPI/gTargetDPI;

var gMax = 0;


Main();

function getTextRanges ( item ) {
		
	var tsrsArray = [];
	
	if (item.parentPage == null) return false;
	
	if (item.texts.length == 0) {
		$.write("Keine texts.length für " + item.contents + "\r");
	}

	for (var i = 0; i < item.texts.length && i < 2; i++) {

		var txt = item.texts[i];
		var tsrs = txt.textStyleRanges;
		if (tsrs.length > 0) {
			for (var j = 0; j < tsrs.length; j++) {
				var tsr = tsrs[j];
				obj = {
					app_font : tsr.appliedFont,
					letterSpacing: tsr.tracking,
					font : tsr.appliedFont.postscriptName,
					size: round_down(tsr.pointSize),
					textFrame: item 
				}
				if (tsr.characters.length > 1) {
					tsrsArray.push(obj);
				} else {
					$.write( "Zu wenig Zeichen: " + tsr.contents  + "\r");
				}
			}
		} else {
			$.write("Kein textStyleRange: " + item.contents +"\r" );
		}
	}
	
	return tsrsArray;

}


function sortArrayByKeys () {
	
}


function Main() {

	alert("Das Skript dauert etwas.\rSchritt 1: Suche alle Textfelder.");
	
	var allTextFrames = [];

	var allframes = app.activeDocument.allPageItems;
	for (i=0; i<allframes.length; i++) {
		if (allframes[i] instanceof TextFrame)
			allTextFrames.push(allframes[i]);
	}

	$.write("total TextFrames: " + allTextFrames.length + "\r");

	alert(allTextFrames.length + " Textfelder gefunden.\rSchritt 2: Stil-Varianten ermitteln.");

	var textRanges = [];
	
	for (var i = 0; i < allTextFrames.length && (gMax == 0 || i < gMax); i++) {
		var item = allTextFrames[i];
		var tsrs = getTextRanges(item);
		if (tsrs) textRanges = textRanges.concat( getTextRanges(item) );
		item.gradientFillAngle = 0;
	}
	 
	var textRangesByType = {};
	
	for (var j = 0; j < textRanges.length; j++) {
		var obj = textRanges[j];
		if (textRangesByType[obj.font] == null) textRangesByType[obj.font] = {};
		if (textRangesByType[obj.font][obj.size] == null) textRangesByType[obj.font][obj.size] = {};
		if (textRangesByType[obj.font][obj.size][obj.letterSpacing] == null) textRangesByType[obj.font][obj.size][obj.letterSpacing] = [];
		textRangesByType[obj.font][obj.size][obj.letterSpacing].push( obj );
	}
  

	var newDoc = app.documents.add();
	
	newDoc.pages.item(0).marginPreferences.properties = {right:0, left:0, bottom:0, top:0};
	
	newDoc.viewPreferences.horizontalMeasurementUnits = gDoc.viewPreferences.horizontalMeasurementUnits;
	newDoc.viewPreferences.verticalMeasurementUnits = gDoc.viewPreferences.verticalMeasurementUnits;
	newDoc.documentPreferences.properties = {
		pageHeight: '900px',
		pageWidth: '900px'
	}

	var page = newDoc.pages.item(0);
	
/*
	newDoc.pages.item(0).resize(
		CoordinateSpaces.INNER_COORDINATES,
		AnchorPoint.CENTER_ANCHOR,
		ResizeMethods.REPLACING_CURRENT_DIMENSIONS_WITH,
		[gTypeSheetDimensions.width, gTypeSheetDimensions.height]
	);
   */

	alert("Schritt 3: Type-Sheet erstellen.");

	var y = 100;
	var typeCount = 0;
	var firstobj;

	for (var name in textRangesByType) {
		   var obj = textRangesByType[name];
			
		   var tf = page.textFrames.add();
			
		   tf.geometricBounds = [y,50,y+20,200];
		   tf.contents = name;
		   tf.move([50,y]);
   
			var keys = [];
			for (var k in obj) {
				keys.push(k);
			}
			keys.sort();
			
			var skeys = [];
			for (var k in obj[keys[0]]) {
				skeys.push(k);
			}
		
			var idPrefix = String.fromCharCode(65+typeCount);
			
			firstobj = obj[keys[0]][skeys[0]][0];
			tf.paragraphs.everyItem().appliedFont = firstobj.app_font;
			
			for (var n = 0; n < keys.length; n++) {
				
					var size = keys[n];

					var stf = page.textFrames.add();
					stf.geometricBounds = [y,50,y+(size*1)+20,500];
					stf.move([200,y]);
					stf.contents = size + " pt / Android: " + calc_target_size(size) + " dp";

					stf.paragraphs.everyItem().appliedFont = firstobj.app_font;
					stf.texts[0].pointSize = size;

					var skeys = [];
					for (var k in obj[size]) {
						skeys.push(k);
					}
					skeys.sort();
			
					for (var m = 0; m < skeys.length; m++) {
						 var spacing = skeys[m];
						 var tobj = obj[size][spacing][0];
						
						 var ptf = page.textFrames.add();
						  ptf.geometricBounds = [y,50,y+20,200];
						  ptf.move([600,y]);
						  ptf.contents = " Letter-Spacing: " + spacing;
						 
						  ptf.paragraphs.everyItem().appliedFont = tobj.app_font;
						  ptf.texts[0].tracking = spacing*1;
				  
						var idtf = page.textFrames.add();
						idtf.geometricBounds = [y,0,y+20,80];
						idtf.move([800,y]);
						
						var id = idPrefix+size+ "-"+spacing;
						idtf.contents = "[ "+id+" ] / "+obj[size][spacing].length;
						
						for (var z = 0; z < obj[size][spacing].length; z++) {
							obj[size][spacing][z].id = id;
						}
					
						 y += 15;
					}
					
					y += (size*1)+5;
			}
/*
			var p = tf.parentStory.paragraphs.item(0);
		   p.appliedFont = obj.app_fonts;
*/
		   
			
		   y += 30;
		   typeCount++;
		   
	}
	
	alert("Schritt 4: Alle Textfelder mit Etiketten bestücken.\rDas kann dauern ...");

	for (var name in textRangesByType) {
		for (var size in textRangesByType[name]) {
			for (var spacing in textRangesByType[name][size]) {
				$.write(name + "  " + size + "\r");
				for (var i in textRangesByType[name][size][spacing]) {
					var obj = textRangesByType[name][size][spacing][i];
					
					var item = obj.textFrame;
									
					if (item.parentPage != null) {
						var tf = gDoc.textFrames.add();
						tf.geometricBounds = [0,0,25,60];
						tf.move(gTagLayer);
						tf.move(item.parentPage);
		
						var bounds = item.geometricBounds;
						var width = bounds[3]-bounds[1];
						
						tf.move([bounds[1]+item.gradientFillAngle*70+width/2-30, bounds[0]-10]);
						tf.contents = obj.id;
						
						var ps = tf.paragraphs.everyItem();
						ps.justification = Justification.CENTER_ALIGN;
						ps.pointSize = 12;
						ps.appliedFont = firstobj.app_font;
						ps.fillColor = gLabelColor;
						ps.baselineShift = -5;
						tf.fillColor = gTagColor;
						
						item.gradientFillAngle++;
					} else {
						$.write("Kein parentPage für " + item.contents + "\r");
					}

				}
			}
		}
	}
   
	alert("Fertig!");

	newDoc.pages.item(0).resize(
		CoordinateSpaces.INNER_COORDINATES,
		AnchorPoint.TOP_LEFT_ANCHOR,
		ResizeMethods.REPLACING_CURRENT_DIMENSIONS_WITH,
		[900, y+100]
	);
	
	
}

function calc_target_size ( n ) {
	return Math.ceil( n * gRatioDPI );
}

function round_down (n) {
		return Math.floor( n );
}


function ErrorExit(error, icon) {
	alert(error, gScriptName + " - " + gScriptVersion, icon);
	exit();
}


function var_dump (obj) {

	var str = "";
	for (var j in obj) {
		str += j + " = " + (obj[j]+"").substr(0,20) + "\n";
	}
	$.write(str);
}

