//
// Colors
// This script provides a table of name colors. The source list is from
// the X11 rgb.txt file.
//
// $Id: Colors.jsx,v 1.3 2010/03/29 02:23:23 anonymous Exp $
// Copyright: (c)2005, xbytor
// License: http://www.opensource.org/licenses/bsd-license.php
// Contact: xbytor@gmail.com
//

Colors = new function() {};

Colors.generateTable = function(infile, outfile) {
  infile.open("r") || throwFileError("Unable to open input file ");
  var str = infile.read();
  infile.close();

  outfile.open("w") || throwFileError("Unable to open output file ");
  
  var ostr = '';
  var lines = str.split(/\n/);

  // skip the first line
  for (var i = 1; i < lines.length; i++) {
    var line = lines[i].split(/\s+/);
    var r = line.shift();
    if (r == '') {
      r = line.shift();
    }
    var g = line.shift();
    var b = line.shift();
    var nm = line.join(' ');

    ostr += ('Colors.addColor("' + nm + '", ' +
             r + ', ' + g + ', ' + b + ");\n");
  }

  outfile.write(ostr);
  outfile.close();
};

Colors.createRGBColor = function (r, g, b) {
  var c = new RGBColor();
  c.red = r; c.green = g; c.blue = b;
  var sc = new SolidColor();
  sc.rgb = c;
  return sc;
};

Colors.generate = function() {
  Colors.generateTable(new File("/c/work/xtools/xlib/rgb.txt"),
                       new File("/c/tmp/rgb.jsx"));
};

Colors.addColor = function(nm, r, g, b) {
  Colors.colorMap[nm] = Colors.createRGBColor(r, g, b);
  Colors.colorList.push(Colors.colorMap[nm]);
};

Colors.colorMap = {};
Colors.colorList = [];

// This is the primary external API
Colors.getColor = function(nm) {
  var c = Colors.colorMap[nm];
  if (!c) {
    c = Colors.colorMap[nm.toLowerCase()];
  }
  return c;
};

Colors.getRandomColor = function() {
   var len = Colors.colorList.length;

   return Colors.colorList[Math.floor(Math.random(len) * len)];
};


// Uncomment this to generate the colors for the color table
//Colors.generate();

// insert generated table here...
Colors.addColor("snow", 255, 250, 250);
Colors.addColor("ghost white", 248, 248, 255);
Colors.addColor("GhostWhite", 248, 248, 255);
Colors.addColor("white smoke", 245, 245, 245);
Colors.addColor("WhiteSmoke", 245, 245, 245);
Colors.addColor("gainsboro", 220, 220, 220);
Colors.addColor("floral white", 255, 250, 240);
Colors.addColor("FloralWhite", 255, 250, 240);
Colors.addColor("old lace", 253, 245, 230);
Colors.addColor("OldLace", 253, 245, 230);
Colors.addColor("linen", 250, 240, 230);
Colors.addColor("antique white", 250, 235, 215);
Colors.addColor("AntiqueWhite", 250, 235, 215);
Colors.addColor("papaya whip", 255, 239, 213);
Colors.addColor("PapayaWhip", 255, 239, 213);
Colors.addColor("blanched almond", 255, 235, 205);
Colors.addColor("BlanchedAlmond", 255, 235, 205);
Colors.addColor("bisque", 255, 228, 196);
Colors.addColor("peach puff", 255, 218, 185);
Colors.addColor("PeachPuff", 255, 218, 185);
Colors.addColor("navajo white", 255, 222, 173);
Colors.addColor("NavajoWhite", 255, 222, 173);
Colors.addColor("moccasin", 255, 228, 181);
Colors.addColor("cornsilk", 255, 248, 220);
Colors.addColor("ivory", 255, 255, 240);
Colors.addColor("lemon chiffon", 255, 250, 205);
Colors.addColor("LemonChiffon", 255, 250, 205);
Colors.addColor("seashell", 255, 245, 238);
Colors.addColor("honeydew", 240, 255, 240);
Colors.addColor("mint cream", 245, 255, 250);
Colors.addColor("MintCream", 245, 255, 250);
Colors.addColor("azure", 240, 255, 255);
Colors.addColor("alice blue", 240, 248, 255);
Colors.addColor("AliceBlue", 240, 248, 255);
Colors.addColor("lavender", 230, 230, 250);
Colors.addColor("lavender blush", 255, 240, 245);
Colors.addColor("LavenderBlush", 255, 240, 245);
Colors.addColor("misty rose", 255, 228, 225);
Colors.addColor("MistyRose", 255, 228, 225);
Colors.addColor("white", 255, 255, 255);
Colors.addColor("black", 0, 0, 0);
Colors.addColor("dark slate gray", 47, 79, 79);
Colors.addColor("DarkSlateGray", 47, 79, 79);
Colors.addColor("dark slate grey", 47, 79, 79);
Colors.addColor("DarkSlateGrey", 47, 79, 79);
Colors.addColor("dim gray", 105, 105, 105);
Colors.addColor("DimGray", 105, 105, 105);
Colors.addColor("dim grey", 105, 105, 105);
Colors.addColor("DimGrey", 105, 105, 105);
Colors.addColor("slate gray", 112, 128, 144);
Colors.addColor("SlateGray", 112, 128, 144);
Colors.addColor("slate grey", 112, 128, 144);
Colors.addColor("SlateGrey", 112, 128, 144);
Colors.addColor("light slate gray", 119, 136, 153);
Colors.addColor("LightSlateGray", 119, 136, 153);
Colors.addColor("light slate grey", 119, 136, 153);
Colors.addColor("LightSlateGrey", 119, 136, 153);
Colors.addColor("gray", 190, 190, 190);
Colors.addColor("grey", 190, 190, 190);
Colors.addColor("light grey", 211, 211, 211);
Colors.addColor("LightGrey", 211, 211, 211);
Colors.addColor("light gray", 211, 211, 211);
Colors.addColor("LightGray", 211, 211, 211);
Colors.addColor("midnight blue", 25, 25, 112);
Colors.addColor("MidnightBlue", 25, 25, 112);
Colors.addColor("navy", 0, 0, 128);
Colors.addColor("navy blue", 0, 0, 128);
Colors.addColor("NavyBlue", 0, 0, 128);
Colors.addColor("cornflower blue", 100, 149, 237);
Colors.addColor("CornflowerBlue", 100, 149, 237);
Colors.addColor("dark slate blue", 72, 61, 139);
Colors.addColor("DarkSlateBlue", 72, 61, 139);
Colors.addColor("slate blue", 106, 90, 205);
Colors.addColor("SlateBlue", 106, 90, 205);
Colors.addColor("medium slate blue", 123, 104, 238);
Colors.addColor("MediumSlateBlue", 123, 104, 238);
Colors.addColor("light slate blue", 132, 112, 255);
Colors.addColor("LightSlateBlue", 132, 112, 255);
Colors.addColor("medium blue", 0, 0, 205);
Colors.addColor("MediumBlue", 0, 0, 205);
Colors.addColor("royal blue", 65, 105, 225);
Colors.addColor("RoyalBlue", 65, 105, 225);
Colors.addColor("blue", 0, 0, 255);
Colors.addColor("dodger blue", 30, 144, 255);
Colors.addColor("DodgerBlue", 30, 144, 255);
Colors.addColor("deep sky blue", 0, 191, 255);
Colors.addColor("DeepSkyBlue", 0, 191, 255);
Colors.addColor("sky blue", 135, 206, 235);
Colors.addColor("SkyBlue", 135, 206, 235);
Colors.addColor("light sky blue", 135, 206, 250);
Colors.addColor("LightSkyBlue", 135, 206, 250);
Colors.addColor("steel blue", 70, 130, 180);
Colors.addColor("SteelBlue", 70, 130, 180);
Colors.addColor("light steel blue", 176, 196, 222);
Colors.addColor("LightSteelBlue", 176, 196, 222);
Colors.addColor("light blue", 173, 216, 230);
Colors.addColor("LightBlue", 173, 216, 230);
Colors.addColor("powder blue", 176, 224, 230);
Colors.addColor("PowderBlue", 176, 224, 230);
Colors.addColor("pale turquoise", 175, 238, 238);
Colors.addColor("PaleTurquoise", 175, 238, 238);
Colors.addColor("dark turquoise", 0, 206, 209);
Colors.addColor("DarkTurquoise", 0, 206, 209);
Colors.addColor("medium turquoise", 72, 209, 204);
Colors.addColor("MediumTurquoise", 72, 209, 204);
Colors.addColor("turquoise", 64, 224, 208);
Colors.addColor("cyan", 0, 255, 255);
Colors.addColor("light cyan", 224, 255, 255);
Colors.addColor("LightCyan", 224, 255, 255);
Colors.addColor("cadet blue", 95, 158, 160);
Colors.addColor("CadetBlue", 95, 158, 160);
Colors.addColor("medium aquamarine", 102, 205, 170);
Colors.addColor("MediumAquamarine", 102, 205, 170);
Colors.addColor("aquamarine", 127, 255, 212);
Colors.addColor("dark green", 0, 100, 0);
Colors.addColor("DarkGreen", 0, 100, 0);
Colors.addColor("dark olive green", 85, 107, 47);
Colors.addColor("DarkOliveGreen", 85, 107, 47);
Colors.addColor("dark sea green", 143, 188, 143);
Colors.addColor("DarkSeaGreen", 143, 188, 143);
Colors.addColor("sea green", 46, 139, 87);
Colors.addColor("SeaGreen", 46, 139, 87);
Colors.addColor("medium sea green", 60, 179, 113);
Colors.addColor("MediumSeaGreen", 60, 179, 113);
Colors.addColor("light sea green", 32, 178, 170);
Colors.addColor("LightSeaGreen", 32, 178, 170);
Colors.addColor("pale green", 152, 251, 152);
Colors.addColor("PaleGreen", 152, 251, 152);
Colors.addColor("spring green", 0, 255, 127);
Colors.addColor("SpringGreen", 0, 255, 127);
Colors.addColor("lawn green", 124, 252, 0);
Colors.addColor("LawnGreen", 124, 252, 0);
Colors.addColor("green", 0, 255, 0);
Colors.addColor("chartreuse", 127, 255, 0);
Colors.addColor("medium spring green", 0, 250, 154);
Colors.addColor("MediumSpringGreen", 0, 250, 154);
Colors.addColor("green yellow", 173, 255, 47);
Colors.addColor("GreenYellow", 173, 255, 47);
Colors.addColor("lime green", 50, 205, 50);
Colors.addColor("LimeGreen", 50, 205, 50);
Colors.addColor("yellow green", 154, 205, 50);
Colors.addColor("YellowGreen", 154, 205, 50);
Colors.addColor("forest green", 34, 139, 34);
Colors.addColor("ForestGreen", 34, 139, 34);
Colors.addColor("olive drab", 107, 142, 35);
Colors.addColor("OliveDrab", 107, 142, 35);
Colors.addColor("dark khaki", 189, 183, 107);
Colors.addColor("DarkKhaki", 189, 183, 107);
Colors.addColor("khaki", 240, 230, 140);
Colors.addColor("pale goldenrod", 238, 232, 170);
Colors.addColor("PaleGoldenrod", 238, 232, 170);
Colors.addColor("light goldenrod yellow", 250, 250, 210);
Colors.addColor("LightGoldenrodYellow", 250, 250, 210);
Colors.addColor("light yellow", 255, 255, 224);
Colors.addColor("LightYellow", 255, 255, 224);
Colors.addColor("yellow", 255, 255, 0);
Colors.addColor("gold", 255, 215, 0);
Colors.addColor("light goldenrod", 238, 221, 130);
Colors.addColor("LightGoldenrod", 238, 221, 130);
Colors.addColor("goldenrod", 218, 165, 32);
Colors.addColor("dark goldenrod", 184, 134, 11);
Colors.addColor("DarkGoldenrod", 184, 134, 11);
Colors.addColor("rosy brown", 188, 143, 143);
Colors.addColor("RosyBrown", 188, 143, 143);
Colors.addColor("indian red", 205, 92, 92);
Colors.addColor("IndianRed", 205, 92, 92);
Colors.addColor("saddle brown", 139, 69, 19);
Colors.addColor("SaddleBrown", 139, 69, 19);
Colors.addColor("sienna", 160, 82, 45);
Colors.addColor("peru", 205, 133, 63);
Colors.addColor("burlywood", 222, 184, 135);
Colors.addColor("beige", 245, 245, 220);
Colors.addColor("wheat", 245, 222, 179);
Colors.addColor("sandy brown", 244, 164, 96);
Colors.addColor("SandyBrown", 244, 164, 96);
Colors.addColor("tan", 210, 180, 140);
Colors.addColor("chocolate", 210, 105, 30);
Colors.addColor("firebrick", 178, 34, 34);
Colors.addColor("brown", 165, 42, 42);
Colors.addColor("dark salmon", 233, 150, 122);
Colors.addColor("DarkSalmon", 233, 150, 122);
Colors.addColor("salmon", 250, 128, 114);
Colors.addColor("light salmon", 255, 160, 122);
Colors.addColor("LightSalmon", 255, 160, 122);
Colors.addColor("orange", 255, 165, 0);
Colors.addColor("dark orange", 255, 140, 0);
Colors.addColor("DarkOrange", 255, 140, 0);
Colors.addColor("coral", 255, 127, 80);
Colors.addColor("light coral", 240, 128, 128);
Colors.addColor("LightCoral", 240, 128, 128);
Colors.addColor("tomato", 255, 99, 71);
Colors.addColor("orange red", 255, 69, 0);
Colors.addColor("OrangeRed", 255, 69, 0);
Colors.addColor("red", 255, 0, 0);
Colors.addColor("hot pink", 255, 105, 180);
Colors.addColor("HotPink", 255, 105, 180);
Colors.addColor("deep pink", 255, 20, 147);
Colors.addColor("DeepPink", 255, 20, 147);
Colors.addColor("pink", 255, 192, 203);
Colors.addColor("light pink", 255, 182, 193);
Colors.addColor("LightPink", 255, 182, 193);
Colors.addColor("pale violet red", 219, 112, 147);
Colors.addColor("PaleVioletRed", 219, 112, 147);
Colors.addColor("maroon", 176, 48, 96);
Colors.addColor("medium violet red", 199, 21, 133);
Colors.addColor("MediumVioletRed", 199, 21, 133);
Colors.addColor("violet red", 208, 32, 144);
Colors.addColor("VioletRed", 208, 32, 144);
Colors.addColor("magenta", 255, 0, 255);
Colors.addColor("violet", 238, 130, 238);
Colors.addColor("plum", 221, 160, 221);
Colors.addColor("orchid", 218, 112, 214);
Colors.addColor("medium orchid", 186, 85, 211);
Colors.addColor("MediumOrchid", 186, 85, 211);
Colors.addColor("dark orchid", 153, 50, 204);
Colors.addColor("DarkOrchid", 153, 50, 204);
Colors.addColor("dark violet", 148, 0, 211);
Colors.addColor("DarkViolet", 148, 0, 211);
Colors.addColor("blue violet", 138, 43, 226);
Colors.addColor("BlueViolet", 138, 43, 226);
Colors.addColor("purple", 160, 32, 240);
Colors.addColor("medium purple", 147, 112, 219);
Colors.addColor("MediumPurple", 147, 112, 219);
Colors.addColor("thistle", 216, 191, 216);
Colors.addColor("snow1", 255, 250, 250);
Colors.addColor("snow2", 238, 233, 233);
Colors.addColor("snow3", 205, 201, 201);
Colors.addColor("snow4", 139, 137, 137);
Colors.addColor("seashell1", 255, 245, 238);
Colors.addColor("seashell2", 238, 229, 222);
Colors.addColor("seashell3", 205, 197, 191);
Colors.addColor("seashell4", 139, 134, 130);
Colors.addColor("AntiqueWhite1", 255, 239, 219);
Colors.addColor("AntiqueWhite2", 238, 223, 204);
Colors.addColor("AntiqueWhite3", 205, 192, 176);
Colors.addColor("AntiqueWhite4", 139, 131, 120);
Colors.addColor("bisque1", 255, 228, 196);
Colors.addColor("bisque2", 238, 213, 183);
Colors.addColor("bisque3", 205, 183, 158);
Colors.addColor("bisque4", 139, 125, 107);
Colors.addColor("PeachPuff1", 255, 218, 185);
Colors.addColor("PeachPuff2", 238, 203, 173);
Colors.addColor("PeachPuff3", 205, 175, 149);
Colors.addColor("PeachPuff4", 139, 119, 101);
Colors.addColor("NavajoWhite1", 255, 222, 173);
Colors.addColor("NavajoWhite2", 238, 207, 161);
Colors.addColor("NavajoWhite3", 205, 179, 139);
Colors.addColor("NavajoWhite4", 139, 121, 94);
Colors.addColor("LemonChiffon1", 255, 250, 205);
Colors.addColor("LemonChiffon2", 238, 233, 191);
Colors.addColor("LemonChiffon3", 205, 201, 165);
Colors.addColor("LemonChiffon4", 139, 137, 112);
Colors.addColor("cornsilk1", 255, 248, 220);
Colors.addColor("cornsilk2", 238, 232, 205);
Colors.addColor("cornsilk3", 205, 200, 177);
Colors.addColor("cornsilk4", 139, 136, 120);
Colors.addColor("ivory1", 255, 255, 240);
Colors.addColor("ivory2", 238, 238, 224);
Colors.addColor("ivory3", 205, 205, 193);
Colors.addColor("ivory4", 139, 139, 131);
Colors.addColor("honeydew1", 240, 255, 240);
Colors.addColor("honeydew2", 224, 238, 224);
Colors.addColor("honeydew3", 193, 205, 193);
Colors.addColor("honeydew4", 131, 139, 131);
Colors.addColor("LavenderBlush1", 255, 240, 245);
Colors.addColor("LavenderBlush2", 238, 224, 229);
Colors.addColor("LavenderBlush3", 205, 193, 197);
Colors.addColor("LavenderBlush4", 139, 131, 134);
Colors.addColor("MistyRose1", 255, 228, 225);
Colors.addColor("MistyRose2", 238, 213, 210);
Colors.addColor("MistyRose3", 205, 183, 181);
Colors.addColor("MistyRose4", 139, 125, 123);
Colors.addColor("azure1", 240, 255, 255);
Colors.addColor("azure2", 224, 238, 238);
Colors.addColor("azure3", 193, 205, 205);
Colors.addColor("azure4", 131, 139, 139);
Colors.addColor("SlateBlue1", 131, 111, 255);
Colors.addColor("SlateBlue2", 122, 103, 238);
Colors.addColor("SlateBlue3", 105, 89, 205);
Colors.addColor("SlateBlue4", 71, 60, 139);
Colors.addColor("RoyalBlue1", 72, 118, 255);
Colors.addColor("RoyalBlue2", 67, 110, 238);
Colors.addColor("RoyalBlue3", 58, 95, 205);
Colors.addColor("RoyalBlue4", 39, 64, 139);
Colors.addColor("blue1", 0, 0, 255);
Colors.addColor("blue2", 0, 0, 238);
Colors.addColor("blue3", 0, 0, 205);
Colors.addColor("blue4", 0, 0, 139);
Colors.addColor("DodgerBlue1", 30, 144, 255);
Colors.addColor("DodgerBlue2", 28, 134, 238);
Colors.addColor("DodgerBlue3", 24, 116, 205);
Colors.addColor("DodgerBlue4", 16, 78, 139);
Colors.addColor("SteelBlue1", 99, 184, 255);
Colors.addColor("SteelBlue2", 92, 172, 238);
Colors.addColor("SteelBlue3", 79, 148, 205);
Colors.addColor("SteelBlue4", 54, 100, 139);
Colors.addColor("DeepSkyBlue1", 0, 191, 255);
Colors.addColor("DeepSkyBlue2", 0, 178, 238);
Colors.addColor("DeepSkyBlue3", 0, 154, 205);
Colors.addColor("DeepSkyBlue4", 0, 104, 139);
Colors.addColor("SkyBlue1", 135, 206, 255);
Colors.addColor("SkyBlue2", 126, 192, 238);
Colors.addColor("SkyBlue3", 108, 166, 205);
Colors.addColor("SkyBlue4", 74, 112, 139);
Colors.addColor("LightSkyBlue1", 176, 226, 255);
Colors.addColor("LightSkyBlue2", 164, 211, 238);
Colors.addColor("LightSkyBlue3", 141, 182, 205);
Colors.addColor("LightSkyBlue4", 96, 123, 139);
Colors.addColor("SlateGray1", 198, 226, 255);
Colors.addColor("SlateGray2", 185, 211, 238);
Colors.addColor("SlateGray3", 159, 182, 205);
Colors.addColor("SlateGray4", 108, 123, 139);
Colors.addColor("LightSteelBlue1", 202, 225, 255);
Colors.addColor("LightSteelBlue2", 188, 210, 238);
Colors.addColor("LightSteelBlue3", 162, 181, 205);
Colors.addColor("LightSteelBlue4", 110, 123, 139);
Colors.addColor("LightBlue1", 191, 239, 255);
Colors.addColor("LightBlue2", 178, 223, 238);
Colors.addColor("LightBlue3", 154, 192, 205);
Colors.addColor("LightBlue4", 104, 131, 139);
Colors.addColor("LightCyan1", 224, 255, 255);
Colors.addColor("LightCyan2", 209, 238, 238);
Colors.addColor("LightCyan3", 180, 205, 205);
Colors.addColor("LightCyan4", 122, 139, 139);
Colors.addColor("PaleTurquoise1", 187, 255, 255);
Colors.addColor("PaleTurquoise2", 174, 238, 238);
Colors.addColor("PaleTurquoise3", 150, 205, 205);
Colors.addColor("PaleTurquoise4", 102, 139, 139);
Colors.addColor("CadetBlue1", 152, 245, 255);
Colors.addColor("CadetBlue2", 142, 229, 238);
Colors.addColor("CadetBlue3", 122, 197, 205);
Colors.addColor("CadetBlue4", 83, 134, 139);
Colors.addColor("turquoise1", 0, 245, 255);
Colors.addColor("turquoise2", 0, 229, 238);
Colors.addColor("turquoise3", 0, 197, 205);
Colors.addColor("turquoise4", 0, 134, 139);
Colors.addColor("cyan1", 0, 255, 255);
Colors.addColor("cyan2", 0, 238, 238);
Colors.addColor("cyan3", 0, 205, 205);
Colors.addColor("cyan4", 0, 139, 139);
Colors.addColor("DarkSlateGray1", 151, 255, 255);
Colors.addColor("DarkSlateGray2", 141, 238, 238);
Colors.addColor("DarkSlateGray3", 121, 205, 205);
Colors.addColor("DarkSlateGray4", 82, 139, 139);
Colors.addColor("aquamarine1", 127, 255, 212);
Colors.addColor("aquamarine2", 118, 238, 198);
Colors.addColor("aquamarine3", 102, 205, 170);
Colors.addColor("aquamarine4", 69, 139, 116);
Colors.addColor("DarkSeaGreen1", 193, 255, 193);
Colors.addColor("DarkSeaGreen2", 180, 238, 180);
Colors.addColor("DarkSeaGreen3", 155, 205, 155);
Colors.addColor("DarkSeaGreen4", 105, 139, 105);
Colors.addColor("SeaGreen1", 84, 255, 159);
Colors.addColor("SeaGreen2", 78, 238, 148);
Colors.addColor("SeaGreen3", 67, 205, 128);
Colors.addColor("SeaGreen4", 46, 139, 87);
Colors.addColor("PaleGreen1", 154, 255, 154);
Colors.addColor("PaleGreen2", 144, 238, 144);
Colors.addColor("PaleGreen3", 124, 205, 124);
Colors.addColor("PaleGreen4", 84, 139, 84);
Colors.addColor("SpringGreen1", 0, 255, 127);
Colors.addColor("SpringGreen2", 0, 238, 118);
Colors.addColor("SpringGreen3", 0, 205, 102);
Colors.addColor("SpringGreen4", 0, 139, 69);
Colors.addColor("green1", 0, 255, 0);
Colors.addColor("green2", 0, 238, 0);
Colors.addColor("green3", 0, 205, 0);
Colors.addColor("green4", 0, 139, 0);
Colors.addColor("chartreuse1", 127, 255, 0);
Colors.addColor("chartreuse2", 118, 238, 0);
Colors.addColor("chartreuse3", 102, 205, 0);
Colors.addColor("chartreuse4", 69, 139, 0);
Colors.addColor("OliveDrab1", 192, 255, 62);
Colors.addColor("OliveDrab2", 179, 238, 58);
Colors.addColor("OliveDrab3", 154, 205, 50);
Colors.addColor("OliveDrab4", 105, 139, 34);
Colors.addColor("DarkOliveGreen1", 202, 255, 112);
Colors.addColor("DarkOliveGreen2", 188, 238, 104);
Colors.addColor("DarkOliveGreen3", 162, 205, 90);
Colors.addColor("DarkOliveGreen4", 110, 139, 61);
Colors.addColor("khaki1", 255, 246, 143);
Colors.addColor("khaki2", 238, 230, 133);
Colors.addColor("khaki3", 205, 198, 115);
Colors.addColor("khaki4", 139, 134, 78);
Colors.addColor("LightGoldenrod1", 255, 236, 139);
Colors.addColor("LightGoldenrod2", 238, 220, 130);
Colors.addColor("LightGoldenrod3", 205, 190, 112);
Colors.addColor("LightGoldenrod4", 139, 129, 76);
Colors.addColor("LightYellow1", 255, 255, 224);
Colors.addColor("LightYellow2", 238, 238, 209);
Colors.addColor("LightYellow3", 205, 205, 180);
Colors.addColor("LightYellow4", 139, 139, 122);
Colors.addColor("yellow1", 255, 255, 0);
Colors.addColor("yellow2", 238, 238, 0);
Colors.addColor("yellow3", 205, 205, 0);
Colors.addColor("yellow4", 139, 139, 0);
Colors.addColor("gold1", 255, 215, 0);
Colors.addColor("gold2", 238, 201, 0);
Colors.addColor("gold3", 205, 173, 0);
Colors.addColor("gold4", 139, 117, 0);
Colors.addColor("goldenrod1", 255, 193, 37);
Colors.addColor("goldenrod2", 238, 180, 34);
Colors.addColor("goldenrod3", 205, 155, 29);
Colors.addColor("goldenrod4", 139, 105, 20);
Colors.addColor("DarkGoldenrod1", 255, 185, 15);
Colors.addColor("DarkGoldenrod2", 238, 173, 14);
Colors.addColor("DarkGoldenrod3", 205, 149, 12);
Colors.addColor("DarkGoldenrod4", 139, 101, 8);
Colors.addColor("RosyBrown1", 255, 193, 193);
Colors.addColor("RosyBrown2", 238, 180, 180);
Colors.addColor("RosyBrown3", 205, 155, 155);
Colors.addColor("RosyBrown4", 139, 105, 105);
Colors.addColor("IndianRed1", 255, 106, 106);
Colors.addColor("IndianRed2", 238, 99, 99);
Colors.addColor("IndianRed3", 205, 85, 85);
Colors.addColor("IndianRed4", 139, 58, 58);
Colors.addColor("sienna1", 255, 130, 71);
Colors.addColor("sienna2", 238, 121, 66);
Colors.addColor("sienna3", 205, 104, 57);
Colors.addColor("sienna4", 139, 71, 38);
Colors.addColor("burlywood1", 255, 211, 155);
Colors.addColor("burlywood2", 238, 197, 145);
Colors.addColor("burlywood3", 205, 170, 125);
Colors.addColor("burlywood4", 139, 115, 85);
Colors.addColor("wheat1", 255, 231, 186);
Colors.addColor("wheat2", 238, 216, 174);
Colors.addColor("wheat3", 205, 186, 150);
Colors.addColor("wheat4", 139, 126, 102);
Colors.addColor("tan1", 255, 165, 79);
Colors.addColor("tan2", 238, 154, 73);
Colors.addColor("tan3", 205, 133, 63);
Colors.addColor("tan4", 139, 90, 43);
Colors.addColor("chocolate1", 255, 127, 36);
Colors.addColor("chocolate2", 238, 118, 33);
Colors.addColor("chocolate3", 205, 102, 29);
Colors.addColor("chocolate4", 139, 69, 19);
Colors.addColor("firebrick1", 255, 48, 48);
Colors.addColor("firebrick2", 238, 44, 44);
Colors.addColor("firebrick3", 205, 38, 38);
Colors.addColor("firebrick4", 139, 26, 26);
Colors.addColor("brown1", 255, 64, 64);
Colors.addColor("brown2", 238, 59, 59);
Colors.addColor("brown3", 205, 51, 51);
Colors.addColor("brown4", 139, 35, 35);
Colors.addColor("salmon1", 255, 140, 105);
Colors.addColor("salmon2", 238, 130, 98);
Colors.addColor("salmon3", 205, 112, 84);
Colors.addColor("salmon4", 139, 76, 57);
Colors.addColor("LightSalmon1", 255, 160, 122);
Colors.addColor("LightSalmon2", 238, 149, 114);
Colors.addColor("LightSalmon3", 205, 129, 98);
Colors.addColor("LightSalmon4", 139, 87, 66);
Colors.addColor("orange1", 255, 165, 0);
Colors.addColor("orange2", 238, 154, 0);
Colors.addColor("orange3", 205, 133, 0);
Colors.addColor("orange4", 139, 90, 0);
Colors.addColor("DarkOrange1", 255, 127, 0);
Colors.addColor("DarkOrange2", 238, 118, 0);
Colors.addColor("DarkOrange3", 205, 102, 0);
Colors.addColor("DarkOrange4", 139, 69, 0);
Colors.addColor("coral1", 255, 114, 86);
Colors.addColor("coral2", 238, 106, 80);
Colors.addColor("coral3", 205, 91, 69);
Colors.addColor("coral4", 139, 62, 47);
Colors.addColor("tomato1", 255, 99, 71);
Colors.addColor("tomato2", 238, 92, 66);
Colors.addColor("tomato3", 205, 79, 57);
Colors.addColor("tomato4", 139, 54, 38);
Colors.addColor("OrangeRed1", 255, 69, 0);
Colors.addColor("OrangeRed2", 238, 64, 0);
Colors.addColor("OrangeRed3", 205, 55, 0);
Colors.addColor("OrangeRed4", 139, 37, 0);
Colors.addColor("red1", 255, 0, 0);
Colors.addColor("red2", 238, 0, 0);
Colors.addColor("red3", 205, 0, 0);
Colors.addColor("red4", 139, 0, 0);
Colors.addColor("DeepPink1", 255, 20, 147);
Colors.addColor("DeepPink2", 238, 18, 137);
Colors.addColor("DeepPink3", 205, 16, 118);
Colors.addColor("DeepPink4", 139, 10, 80);
Colors.addColor("HotPink1", 255, 110, 180);
Colors.addColor("HotPink2", 238, 106, 167);
Colors.addColor("HotPink3", 205, 96, 144);
Colors.addColor("HotPink4", 139, 58, 98);
Colors.addColor("pink1", 255, 181, 197);
Colors.addColor("pink2", 238, 169, 184);
Colors.addColor("pink3", 205, 145, 158);
Colors.addColor("pink4", 139, 99, 108);
Colors.addColor("LightPink1", 255, 174, 185);
Colors.addColor("LightPink2", 238, 162, 173);
Colors.addColor("LightPink3", 205, 140, 149);
Colors.addColor("LightPink4", 139, 95, 101);
Colors.addColor("PaleVioletRed1", 255, 130, 171);
Colors.addColor("PaleVioletRed2", 238, 121, 159);
Colors.addColor("PaleVioletRed3", 205, 104, 137);
Colors.addColor("PaleVioletRed4", 139, 71, 93);
Colors.addColor("maroon1", 255, 52, 179);
Colors.addColor("maroon2", 238, 48, 167);
Colors.addColor("maroon3", 205, 41, 144);
Colors.addColor("maroon4", 139, 28, 98);
Colors.addColor("VioletRed1", 255, 62, 150);
Colors.addColor("VioletRed2", 238, 58, 140);
Colors.addColor("VioletRed3", 205, 50, 120);
Colors.addColor("VioletRed4", 139, 34, 82);
Colors.addColor("magenta1", 255, 0, 255);
Colors.addColor("magenta2", 238, 0, 238);
Colors.addColor("magenta3", 205, 0, 205);
Colors.addColor("magenta4", 139, 0, 139);
Colors.addColor("orchid1", 255, 131, 250);
Colors.addColor("orchid2", 238, 122, 233);
Colors.addColor("orchid3", 205, 105, 201);
Colors.addColor("orchid4", 139, 71, 137);
Colors.addColor("plum1", 255, 187, 255);
Colors.addColor("plum2", 238, 174, 238);
Colors.addColor("plum3", 205, 150, 205);
Colors.addColor("plum4", 139, 102, 139);
Colors.addColor("MediumOrchid1", 224, 102, 255);
Colors.addColor("MediumOrchid2", 209, 95, 238);
Colors.addColor("MediumOrchid3", 180, 82, 205);
Colors.addColor("MediumOrchid4", 122, 55, 139);
Colors.addColor("DarkOrchid1", 191, 62, 255);
Colors.addColor("DarkOrchid2", 178, 58, 238);
Colors.addColor("DarkOrchid3", 154, 50, 205);
Colors.addColor("DarkOrchid4", 104, 34, 139);
Colors.addColor("purple1", 155, 48, 255);
Colors.addColor("purple2", 145, 44, 238);
Colors.addColor("purple3", 125, 38, 205);
Colors.addColor("purple4", 85, 26, 139);
Colors.addColor("MediumPurple1", 171, 130, 255);
Colors.addColor("MediumPurple2", 159, 121, 238);
Colors.addColor("MediumPurple3", 137, 104, 205);
Colors.addColor("MediumPurple4", 93, 71, 139);
Colors.addColor("thistle1", 255, 225, 255);
Colors.addColor("thistle2", 238, 210, 238);
Colors.addColor("thistle3", 205, 181, 205);
Colors.addColor("thistle4", 139, 123, 139);
Colors.addColor("gray0", 0, 0, 0);
Colors.addColor("grey0", 0, 0, 0);
Colors.addColor("gray1", 3, 3, 3);
Colors.addColor("grey1", 3, 3, 3);
Colors.addColor("gray2", 5, 5, 5);
Colors.addColor("grey2", 5, 5, 5);
Colors.addColor("gray3", 8, 8, 8);
Colors.addColor("grey3", 8, 8, 8);
Colors.addColor("gray4", 10, 10, 10);
Colors.addColor("grey4", 10, 10, 10);
Colors.addColor("gray5", 13, 13, 13);
Colors.addColor("grey5", 13, 13, 13);
Colors.addColor("gray6", 15, 15, 15);
Colors.addColor("grey6", 15, 15, 15);
Colors.addColor("gray7", 18, 18, 18);
Colors.addColor("grey7", 18, 18, 18);
Colors.addColor("gray8", 20, 20, 20);
Colors.addColor("grey8", 20, 20, 20);
Colors.addColor("gray9", 23, 23, 23);
Colors.addColor("grey9", 23, 23, 23);
Colors.addColor("gray10", 26, 26, 26);
Colors.addColor("grey10", 26, 26, 26);
Colors.addColor("gray11", 28, 28, 28);
Colors.addColor("grey11", 28, 28, 28);
Colors.addColor("gray12", 31, 31, 31);
Colors.addColor("grey12", 31, 31, 31);
Colors.addColor("gray13", 33, 33, 33);
Colors.addColor("grey13", 33, 33, 33);
Colors.addColor("gray14", 36, 36, 36);
Colors.addColor("grey14", 36, 36, 36);
Colors.addColor("gray15", 38, 38, 38);
Colors.addColor("grey15", 38, 38, 38);
Colors.addColor("gray16", 41, 41, 41);
Colors.addColor("grey16", 41, 41, 41);
Colors.addColor("gray17", 43, 43, 43);
Colors.addColor("grey17", 43, 43, 43);
Colors.addColor("gray18", 46, 46, 46);
Colors.addColor("grey18", 46, 46, 46);
Colors.addColor("gray19", 48, 48, 48);
Colors.addColor("grey19", 48, 48, 48);
Colors.addColor("gray20", 51, 51, 51);
Colors.addColor("grey20", 51, 51, 51);
Colors.addColor("gray21", 54, 54, 54);
Colors.addColor("grey21", 54, 54, 54);
Colors.addColor("gray22", 56, 56, 56);
Colors.addColor("grey22", 56, 56, 56);
Colors.addColor("gray23", 59, 59, 59);
Colors.addColor("grey23", 59, 59, 59);
Colors.addColor("gray24", 61, 61, 61);
Colors.addColor("grey24", 61, 61, 61);
Colors.addColor("gray25", 64, 64, 64);
Colors.addColor("grey25", 64, 64, 64);
Colors.addColor("gray26", 66, 66, 66);
Colors.addColor("grey26", 66, 66, 66);
Colors.addColor("gray27", 69, 69, 69);
Colors.addColor("grey27", 69, 69, 69);
Colors.addColor("gray28", 71, 71, 71);
Colors.addColor("grey28", 71, 71, 71);
Colors.addColor("gray29", 74, 74, 74);
Colors.addColor("grey29", 74, 74, 74);
Colors.addColor("gray30", 77, 77, 77);
Colors.addColor("grey30", 77, 77, 77);
Colors.addColor("gray31", 79, 79, 79);
Colors.addColor("grey31", 79, 79, 79);
Colors.addColor("gray32", 82, 82, 82);
Colors.addColor("grey32", 82, 82, 82);
Colors.addColor("gray33", 84, 84, 84);
Colors.addColor("grey33", 84, 84, 84);
Colors.addColor("gray34", 87, 87, 87);
Colors.addColor("grey34", 87, 87, 87);
Colors.addColor("gray35", 89, 89, 89);
Colors.addColor("grey35", 89, 89, 89);
Colors.addColor("gray36", 92, 92, 92);
Colors.addColor("grey36", 92, 92, 92);
Colors.addColor("gray37", 94, 94, 94);
Colors.addColor("grey37", 94, 94, 94);
Colors.addColor("gray38", 97, 97, 97);
Colors.addColor("grey38", 97, 97, 97);
Colors.addColor("gray39", 99, 99, 99);
Colors.addColor("grey39", 99, 99, 99);
Colors.addColor("gray40", 102, 102, 102);
Colors.addColor("grey40", 102, 102, 102);
Colors.addColor("gray41", 105, 105, 105);
Colors.addColor("grey41", 105, 105, 105);
Colors.addColor("gray42", 107, 107, 107);
Colors.addColor("grey42", 107, 107, 107);
Colors.addColor("gray43", 110, 110, 110);
Colors.addColor("grey43", 110, 110, 110);
Colors.addColor("gray44", 112, 112, 112);
Colors.addColor("grey44", 112, 112, 112);
Colors.addColor("gray45", 115, 115, 115);
Colors.addColor("grey45", 115, 115, 115);
Colors.addColor("gray46", 117, 117, 117);
Colors.addColor("grey46", 117, 117, 117);
Colors.addColor("gray47", 120, 120, 120);
Colors.addColor("grey47", 120, 120, 120);
Colors.addColor("gray48", 122, 122, 122);
Colors.addColor("grey48", 122, 122, 122);
Colors.addColor("gray49", 125, 125, 125);
Colors.addColor("grey49", 125, 125, 125);
Colors.addColor("gray50", 127, 127, 127);
Colors.addColor("grey50", 127, 127, 127);
Colors.addColor("gray51", 130, 130, 130);
Colors.addColor("grey51", 130, 130, 130);
Colors.addColor("gray52", 133, 133, 133);
Colors.addColor("grey52", 133, 133, 133);
Colors.addColor("gray53", 135, 135, 135);
Colors.addColor("grey53", 135, 135, 135);
Colors.addColor("gray54", 138, 138, 138);
Colors.addColor("grey54", 138, 138, 138);
Colors.addColor("gray55", 140, 140, 140);
Colors.addColor("grey55", 140, 140, 140);
Colors.addColor("gray56", 143, 143, 143);
Colors.addColor("grey56", 143, 143, 143);
Colors.addColor("gray57", 145, 145, 145);
Colors.addColor("grey57", 145, 145, 145);
Colors.addColor("gray58", 148, 148, 148);
Colors.addColor("grey58", 148, 148, 148);
Colors.addColor("gray59", 150, 150, 150);
Colors.addColor("grey59", 150, 150, 150);
Colors.addColor("gray60", 153, 153, 153);
Colors.addColor("grey60", 153, 153, 153);
Colors.addColor("gray61", 156, 156, 156);
Colors.addColor("grey61", 156, 156, 156);
Colors.addColor("gray62", 158, 158, 158);
Colors.addColor("grey62", 158, 158, 158);
Colors.addColor("gray63", 161, 161, 161);
Colors.addColor("grey63", 161, 161, 161);
Colors.addColor("gray64", 163, 163, 163);
Colors.addColor("grey64", 163, 163, 163);
Colors.addColor("gray65", 166, 166, 166);
Colors.addColor("grey65", 166, 166, 166);
Colors.addColor("gray66", 168, 168, 168);
Colors.addColor("grey66", 168, 168, 168);
Colors.addColor("gray67", 171, 171, 171);
Colors.addColor("grey67", 171, 171, 171);
Colors.addColor("gray68", 173, 173, 173);
Colors.addColor("grey68", 173, 173, 173);
Colors.addColor("gray69", 176, 176, 176);
Colors.addColor("grey69", 176, 176, 176);
Colors.addColor("gray70", 179, 179, 179);
Colors.addColor("grey70", 179, 179, 179);
Colors.addColor("gray71", 181, 181, 181);
Colors.addColor("grey71", 181, 181, 181);
Colors.addColor("gray72", 184, 184, 184);
Colors.addColor("grey72", 184, 184, 184);
Colors.addColor("gray73", 186, 186, 186);
Colors.addColor("grey73", 186, 186, 186);
Colors.addColor("gray74", 189, 189, 189);
Colors.addColor("grey74", 189, 189, 189);
Colors.addColor("gray75", 191, 191, 191);
Colors.addColor("grey75", 191, 191, 191);
Colors.addColor("gray76", 194, 194, 194);
Colors.addColor("grey76", 194, 194, 194);
Colors.addColor("gray77", 196, 196, 196);
Colors.addColor("grey77", 196, 196, 196);
Colors.addColor("gray78", 199, 199, 199);
Colors.addColor("grey78", 199, 199, 199);
Colors.addColor("gray79", 201, 201, 201);
Colors.addColor("grey79", 201, 201, 201);
Colors.addColor("gray80", 204, 204, 204);
Colors.addColor("grey80", 204, 204, 204);
Colors.addColor("gray81", 207, 207, 207);
Colors.addColor("grey81", 207, 207, 207);
Colors.addColor("gray82", 209, 209, 209);
Colors.addColor("grey82", 209, 209, 209);
Colors.addColor("gray83", 212, 212, 212);
Colors.addColor("grey83", 212, 212, 212);
Colors.addColor("gray84", 214, 214, 214);
Colors.addColor("grey84", 214, 214, 214);
Colors.addColor("gray85", 217, 217, 217);
Colors.addColor("grey85", 217, 217, 217);
Colors.addColor("gray86", 219, 219, 219);
Colors.addColor("grey86", 219, 219, 219);
Colors.addColor("gray87", 222, 222, 222);
Colors.addColor("grey87", 222, 222, 222);
Colors.addColor("gray88", 224, 224, 224);
Colors.addColor("grey88", 224, 224, 224);
Colors.addColor("gray89", 227, 227, 227);
Colors.addColor("grey89", 227, 227, 227);
Colors.addColor("gray90", 229, 229, 229);
Colors.addColor("grey90", 229, 229, 229);
Colors.addColor("gray91", 232, 232, 232);
Colors.addColor("grey91", 232, 232, 232);
Colors.addColor("gray92", 235, 235, 235);
Colors.addColor("grey92", 235, 235, 235);
Colors.addColor("gray93", 237, 237, 237);
Colors.addColor("grey93", 237, 237, 237);
Colors.addColor("gray94", 240, 240, 240);
Colors.addColor("grey94", 240, 240, 240);
Colors.addColor("gray95", 242, 242, 242);
Colors.addColor("grey95", 242, 242, 242);
Colors.addColor("gray96", 245, 245, 245);
Colors.addColor("grey96", 245, 245, 245);
Colors.addColor("gray97", 247, 247, 247);
Colors.addColor("grey97", 247, 247, 247);
Colors.addColor("gray98", 250, 250, 250);
Colors.addColor("grey98", 250, 250, 250);
Colors.addColor("gray99", 252, 252, 252);
Colors.addColor("grey99", 252, 252, 252);
Colors.addColor("gray100", 255, 255, 255);
Colors.addColor("grey100", 255, 255, 255);
Colors.addColor("dark grey", 169, 169, 169);
Colors.addColor("DarkGrey", 169, 169, 169);
Colors.addColor("dark gray", 169, 169, 169);
Colors.addColor("DarkGray", 169, 169, 169);
Colors.addColor("dark blue", 0, 0, 139);
Colors.addColor("DarkBlue", 0, 0, 139);
Colors.addColor("dark cyan", 0, 139, 139);
Colors.addColor("DarkCyan", 0, 139, 139);
Colors.addColor("dark magenta", 139, 0, 139);
Colors.addColor("DarkMagenta", 139, 0, 139);
Colors.addColor("dark red", 139, 0, 0);
Colors.addColor("DarkRed", 139, 0, 0);
Colors.addColor("light green", 144, 238, 144);
Colors.addColor("LightGreen", 144, 238, 144);

"Colors.jsx";
// EOF
